// The existingPosts array is used to track already displayed posts
let existingPosts = [];

// below times are in milliseconds
// duration for slide animations
let duration = 5000;
// refresh rate
let refresh = 30000;

// getUrlParameter helps to fetch URL parameters
function getUrlParameter(name) {
    name = name.replace(/[\[]/, '\\[').replace(/[\]]/, '\\]');
    var regex = new RegExp('[\\?&]' + name + '=([^&#]*)');
    var results = regex.exec(location.search);
    console.log(results)
    return results === null ? '' : decodeURIComponent(results[1].replace(/\+/g, ' '));
}

// secondsAgo calculates how many seconds have passed since the provided date
const secondsAgo = date => Math.floor((new Date() - date) / 1000);

// timeAgo formats the time elapsed in a human readable format
const timeAgo = function(seconds) {
    // An array of intervals for years, months, days, hours, and minutes.
    const intervals = [
        { limit: 31536000, text: 'Jahren' },
        { limit: 2592000, text: 'Monaten' },
        { limit: 86400, text: 'Tagen' },
        { limit: 3600, text: 'Stunden' },
        { limit: 60, text: 'Minuten' }
    ];

    // Loop through the intervals to find which one is the best fit.
    for (let interval of intervals) {
        if (seconds >= interval.limit) {
            return "Vor " + Math.floor(seconds / interval.limit) + ` ${interval.text}`;
        }
    }
    return "Vor " + Math.floor(seconds) + " Sekunden";
};

let includeReplies;

// fetchConfig fetches the configuration from the config.json file
const fetchConfig = async function() {
    try {
        const config = await $.getJSON('config.json');
        $('#navbar-brand').text(config.navbarBrandText);
        $('.navbar').css('background-color', config.navbarColor);
        includeReplies = config.includeReplies;
        return config.defaultServerUrl;
    } catch (error) {
        console.log("Error loading config.json:", error);
        $('#navbar-brand').text("Netzbegrünung Mastowall");
        $('.navbar').css('background-color', "#008939");
        includeReplies = true;
        return "https://gruene.social";
    }
}

// fetchPosts fetches posts from the server using the given hashtag
const fetchPosts = async function(serverUrl, hashtag) {
    try {
        const posts = await $.get(`${serverUrl}/api/v1/timelines/tag/${hashtag}?limit=20`);
        return posts;
    } catch (error) {
        console.error(`Error loading posts for hashtag #${hashtag}:`, error);
    }
};

// updateTimesOnPage updates the time information displayed for each post
const updateTimesOnPage = function() {
    $('.card-text a').each(function() {
        const date = new Date($(this).attr('data-time'));
        const newTimeAgo = timeAgo(secondsAgo(date));
        $(this).text(newTimeAgo);
    });
};

// displayPost creates and displays a post
const displayPost = function(post) {
    if (existingPosts.includes(post.id) || (!includeReplies && post.in_reply_to_id !== null)) return 0;

    existingPosts.push(post.id);

    let cardHTML = `
        <div class="col-sm-3">
            <div class="card m-2 p-2">
                <div class="d-flex align-items-center mb-2">
                    <img src="${post.account.avatar}" class="avatar-img rounded-circle mr-2">
                    <p class="m-0">${DOMPurify.sanitize(post.account.display_name)}</p>
                </div>
                ${post.media_attachments[0] ? `<img src="${post.media_attachments[0].url}" class="card-img-top mb-2">` : ''}
                <p class="card-text">${DOMPurify.sanitize(post.content)}</p>
                ${post.spoiler_text ? `<p class="card-text text-muted spoiler">${DOMPurify.sanitize(post.spoiler_text)}</p>` : ''}
                <p class="card-text text-right"><small class="text-muted"><a href="${post.url}" target="_blank" data-time="${post.created_at}">${timeAgo(secondsAgo(new Date(post.created_at)))}</a></small></p>
            </div>
        </div>
    `;

    let $card = $(cardHTML);
    $('#wall').prepend($card);
    $('.masonry-grid').masonry('prepended', $card);
    return 1;
};

// updateWall displays all posts
const updateWall = function(posts) {
    if (!posts || posts.length === 0) return;

    posts.sort((a, b) => new Date(a.created_at) - new Date(b.created_at));
    let ret = 0
    posts.forEach(post => ret += displayPost(post));
    $('.masonry-grid').masonry('layout');
    return ret;
};


// updateCarousel
const updateCarousel = function(slides, posts) {
    if (!posts || posts.length === 0) return;

    posts.sort((a, b) => new Date(b.created_at) - new Date(a.created_at));
    
    // remove slides in carousel
    slides.innerHTML = "";
    var newHTML = ` <!-- No Indicators -->`
    newHTML += `<!-- the slides -->
                <div class="carousel-inner">
                `
    for( let i = 0; i < posts.length; i++ ) {
        let post = posts[i];
        /*console.log( post.content )*/
        if ( i == 0 ) {
            newHTML += `<div class="carousel-item active" data-mdb-interval="${duration}" data-mdb-pause="false">`;
        }
        else {
            newHTML += `<div class="carousel-item" data-mdb-interval="${duration}" data-mdb-pause="false">`;
        }
        newHTML += `
            <div class="card-big">
                <div class="d-flex align-items-center mb-4">
                    <img src="${post.account.avatar}" class="avatar-img-big rounded-circle mr-4">
                    <p class="avatar-name">${DOMPurify.sanitize(post.account.display_name)}</p>
                </div>
                <hr>
                <div class="row align-items-center vertical-align-center">
                    ${post.media_attachments[0] ? `<div class="col-md-6"><p class="card-text">${DOMPurify.sanitize(post.content)}</p></div>` : `<div class="col-md-12"><p class="card-text">${DOMPurify.sanitize(post.content)}</p></div>`}
                    ${post.media_attachments[0] ? `<div class="col-md-6"><img src="${post.media_attachments[0].url}" class="card-img-bottom" align="center"> </div>` : ''}
                </div>
                <hr>
                <p class="card-text text-right">
                    <small class="text-muted"><a href="${post.url}" target="_blank" data-time="${post.created_at}">${timeAgo(secondsAgo(new Date(post.created_at)))}</a>
                        ${post.favourites_count ? `, <b>${post.favourites_count}</b> mal favorisiert` : '' }
                        ${post.replies_count ? `, <b>${post.replies_count}</b> mal kommentiert` : '' }
                        ${post.reblogs_count ? `, <b>${post.reblogs_count}</b> mal geteilt` : '' }
                    </small>
                </p>
            </div>
        `;
        newHTML += '</div>';
    }
    newHTML += '</div>'
    document.getElementById("myCarousel").innerHTML = newHTML;
};

const showCarousel = function() {
    // show popover
    document.getElementById('popover').style.opacity = '1';
    // Activate Carousel
    $('#myCarousel').carousel("cycle");
}

const hideCarousel = function() {
    // show popover
    document.getElementById('popover').style.opacity = '0';
    // Activate Carousel
}

// hashtagsString returns a single string based on the given array of hashtags
const hashtagsString = function(hashtagsArray) {
  return `${hashtagsArray.map(hashtag => `#${hashtag}`).join(' ')}`;
}

// updateHashtagsOnPage updates the displayed hashtags
const updateHashtagsOnPage = function(hashtagsArray) {
    $('#hashtag-display').text(hashtagsArray.length > 0 ? hashtagsString(hashtagsArray) : 'No hashtags set');
};

// updateHashtagsOnPage updates the document title by appending the given array of hashtags
const updateHashtagsInTitle = function(hashtagsArray) {
    const baseTitle = document.title;
    document.title = `${baseTitle} | ${hashtagsString(hashtagsArray)}`;
}

// handleHashtagDisplayClick handles the event when the hashtag display is clicked
const handleHashtagDisplayClick = function(serverUrl) {
    $('#app-content').addClass('d-none');
    $('#zero-state').removeClass('d-none');

    const currentHashtags = getUrlParameter('hashtags').split(',');
    if ( currentHashtags = null ) {
        currentHasttags = "netzbegruenung,bdk23".split(',')
    }

    for (let i = 0; i < currentHashtags.length; i++) {
        $(`#hashtag${i+1}`).val(currentHashtags[i]);
    }

    $('#serverUrl').val(serverUrl);
};

// handleHashtagFormSubmit handles the submission of the hashtag form
const handleHashtagFormSubmit = function(e, hashtagsArray) {
    e.preventDefault();

    let hashtags = [
        $('#hashtag1').val(),
        $('#hashtag2').val(),
        $('#hashtag3').val()
    ];

    hashtags = hashtags.filter(function(hashtag) {
        return hashtag !== '' && /^[\w]+$/.test(hashtag);
    });

    let serverUrl = $('#serverUrl').val();

    if (!/^https:\/\/[\w.\-]+\/?$/.test(serverUrl)) {
        alert('Invalid server URL.');
        return;
    }

    const newUrl = window.location.origin + window.location.pathname + `?hashtags=${hashtags.join(',')}&server=${serverUrl}`;

    window.location.href = newUrl;
};

// On document ready, the script configures Masonry, handles events, fetches and displays posts
$(document).ready(async function() {
    const defaultServerUrl = await fetchConfig();
    $('.masonry-grid').masonry({
        itemSelector: '.col-sm-3',
        columnWidth: '.col-sm-3',
        percentPosition: true
    });

    setInterval(function() {
        $('.masonry-grid').masonry('layout');
    }, refresh);
    
    let hashtags = getUrlParameter('hashtags');
    if ( hashtags == '' ) {
        hashtags = "netzbegruenung,bdk23";
    }
    const hashtagsArray = hashtags ? hashtags.split(',') : [];
    const serverUrl = getUrlParameter('server') || defaultServerUrl;

    $('#hashtag-display').on('click', function() {
        handleHashtagDisplayClick(serverUrl);
    });

    const slides = $('#myCarousel');
    const popover = $('#popover');

    if (hashtagsArray.length > 0 && hashtagsArray[0] !== '') {
        const allPosts = await Promise.all(hashtagsArray.map(hashtag => fetchPosts(serverUrl, hashtag)));
        updateWall(allPosts.flat());
        setTimeout(function() {
            $('.masonry-grid').masonry('layout');
            }, 2000);

        updateCarousel(slides, allPosts.flat());
        
        setTimeout(async function() { showCarousel(); }, duration)
        setInterval(async function() {
            const newPosts = await Promise.all(hashtagsArray.map(hashtag => fetchPosts(serverUrl, hashtag)));
            let updated = updateWall(newPosts.flat());
            if ( updated > 0 ) {
                hideCarousel()
                updateCarousel(slides, newPosts.flat());
                setTimeout(async function() { showCarousel(); }, duration)
            }
        }, refresh);
    } else {
        $('#zero-state').removeClass('d-none');
        $('#popover').removeClass('popover');
        $('#app-content').addClass('d-none');
    }

    updateHashtagsOnPage(hashtagsArray);
    updateHashtagsInTitle(hashtagsArray);

    $('#hashtag-form').on('submit', function(e) {
        handleHashtagFormSubmit(e, hashtagsArray);
    });


    updateTimesOnPage();
    setInterval(updateTimesOnPage, 60000);
});
